require 'csv'

class Hermes::Admin::BorderingTownsController < Hermes::Admin::BaseController
  before_action :authorize_users
  before_action :check_adminrole
  rescue_from JSON::ParserError, with: :invalid_json

  def index
    query = {}
    ["insee_code", "name"].each do |p|
      query[p]= /#{::Regexp.escape(params[p]).accent_to_regexp}/i unless params[p].blank?
    end 
    @bordering_towns =  Hermes::BorderingTown.readable_by(current_user).where(query).paginate(:page => params[:page])

    @file = {name: 'test'} 
    respond_to do |format|
      format.html
      format.js
    end
  end

  def update_from_file
    file = params[:csv_file]
    return unless file

    allowed_mime_types = ['text/csv', 'application/vnd.ms-excel']

    unless allowed_mime_types.include? file.content_type
      flash[:danger] = 'Invalid file type'
      redirect_to update_from_file_admin_bordering_towns_path and return
    end

    if file.size > 10.megabytes
      flash[:danger] = 'File is too large'
      redirect_to update_from_file_admin_bordering_towns_path and return
    end

    begin
      csv = CSV.parse(File.read(file.to_io), headers: true, col_sep: ",")
    rescue CSV::MalformedCSVError => e
      flash[:danger] = "The file could not be processed: #{e.message}"
      redirect_to update_from_file_admin_bordering_towns_path and return
    end

    doc = Doc.create(
      responsible: current_user.id, 
      type: "file", 
      title: "Fichier de mise à jour des communes limitrophes #{Time.now}.csv",
      blob: csv.to_s,
    )

    UpdateBorderingTownsJob.perform_later(doc.id.to_s)

    flash[:success] = "Mise à jour de #{csv.length} communes en cours. Attention cette mise à jour est asynchrone."
    redirect_to admin_bordering_towns_path
  end


  def show; end

  private

end 