# frozen_string_literal: true

module Hermes
  class SignupUserNotificationPreference
    include Hermes::Concerns::CommonFieldsConcern
    
    field :email, type: String
    field :notification_key, type: String
    field :want_to_be_subscribed, type: Boolean
    field :used_at, type: DateTime
    
    validates_presence_of :email, :notification_key, :want_to_be_subscribed
    owner_field :user_id

    before_save :before_save_actions

    common_indexes *%i[
      email
      notification_key
      want_to_be_subscribed
      used_at
    ]

    def before_save_actions
      self.email = email.strip.downcase unless email.blank? && email_changed?
    end 

    def self.update_user_preference(u)
      NOTIFICATION_KEYS[:user].each do |key, value|
        if pref = Hermes::SignupUserNotificationPreference.order_by(created_at: :desc).find_by(email: u.email.downcase, notification_key: value, used_at: nil)
          pref.want_to_be_subscribed ? u.subscription.subscribe_to_email(value) : u.subscription.unsubscribe_to_email(value)
          pref.update(used_at: DateTime.now)
        end
      end 
    end 
  end
end
