# frozen_string_literal: true

module Hermes
  # HousingRequest serializer
  class HousingRequestFolderSerializer < Hermes::ApplicationSerializer
    type 'housing_request_folders'
 
    attributes :nud_status,
               :consultation_status,
               :application_status,
               :completion,
               :is_still_valid,
               :housing_request_id,
               :required_documents
             

    has_many :required_documents, serializer: RequiredDocumentSerializer do
      object.required_documents
    end

    def nud_status
      object.nud_status    unless @instance_options[:hide_status]
    end

    def consultation_status
      object.consultation_status    unless @instance_options[:hide_status]
    end
    
    def application_status
      object.application_status    unless @instance_options[:hide_status]
    end
    
    def is_still_valid
     true
    end
    
    def housing_request_id
      object.housing_request&.id&.to_s
    end
    
    def required_documents
      object.required_documents.sort_by{|rd| rd.ihm_order}.map do |doc|
        Hermes::RequiredDocumentSerializer.new(
          doc,
          scope: scope,
          root: false,
          base_url: @instance_options[:base_url],
          folder_id: object.id,
          main_applicant_id: object.housing_request&.main_applicant_id,
          hide_status: @instance_options[:hide_status]
        )
      end
    end
  
  end
end
