module BatchTasks
  class SynchroRex < Base

    def initialize(options = {})
      @tagset_id = options[:tagset_id]
      @date = options[:date]
      @client = options[:client] || Hermes::RexClient::Client.new
      @yopmail = options[:yopmail].nil? ? true : options[:yopmail]
      @email_mail_to = options[:email_mail_to] || ['support.trusted@be-ys-trusted-solutions.com']
      @client.set_syncro_rex(self)
    end 

    def client
      @client
    end
    
    def synchronise
      starttime = Time.now
      @log_name = "Synchronisation REX #{print_time(starttime)} with date #{@date}.log"

      add_to_log_file("Starting to update structures")

      @client.traverse_structures({'traceability.lastModifiedDate' => ">#{@date}"}, Proc.new do |rex_entity| 
          begin
            className = (rex_entity['idEntityDefinition'] == @client.ed_entity) ? 'Hermes::Entity' : 'Hermes::Lessor'
            add_to_log_file("Started : Syncro #{className} id_cristal #{rex_entity['fields']['id_cristal']}")

            update_entity(rex_entity, className) if rex_entity['fields']['id_absorbant_cristal'].blank?

            add_to_log_file("Ended : Syncro #{className} id_cristal #{rex_entity['fields']['id_cristal']}")
          rescue => exception
            type = className == 'Hermes::Entity' ? 'Entity' : 'Lessor'
            line = "#{type} id_cristal: #{rex_entity['fields']['id_cristal']} => Error : #{exception}"
            @client.add_error(line)
          end
        end
      )

      add_to_log_file("Starting to update contacts")

      @client.traverse_contacts({'traceability.lastModifiedDate' => ">#{@date}"}, Proc.new do |rex_entity| 
          begin
            add_to_log_file("Started : Syncro Contact id_cristal #{rex_entity['fields']['id_cristal']}")
            update_contact(rex_entity)
            add_to_log_file("Ended : Syncro Contact id_cristal #{rex_entity['fields']['id_cristal']}")
          rescue => exception
            line = "Contact id_cristal: #{rex_entity['fields']['id_cristal']} => Error : #{exception}"
            @client.add_error(line)
          end
        end
      )

      
      add_to_log_file( "REX sync took : #{Time.now - starttime} sec")
      doc = create_log_doc(@tagset_id)
      send_email(doc)
    end 

    def update_entity(rex_entity, className,  arbo_ids = [], already_updated_parent = false)
      rex_entity.stringify_keys!
      fields = rex_entity['fields'].stringify_keys
      return if fields['id_cristal'].blank?

      add_to_log_file("Struture fields #{fields}")
      
      arbo_ids << fields['id_cristal']
      structure = className.constantize.find_by(id_cristal: fields['id_cristal'])
      
      if !structure
        add_to_log_file("Creating new structure #{fields['id_cristal']}")
        structure = className.constantize.create(generate_structure_fields(fields, rex_entity['id']))
        add_to_log_file("End Creating new structure #{fields['id_cristal']}")
      else
        add_to_log_file("Updating existant structure #{fields['id_cristal']} #{structure.id}")
        structure.set(generate_structure_fields(fields, rex_entity['id']).merge(updated_at: DateTime.now))
        add_to_log_file("End Updating existant structure #{fields['id_cristal']} #{structure.id}")
      end
      
      if fields['id_parent_cristal'].blank? 
        structure.parent_id = nil 
      else
        if !arbo_ids.include?(fields['id_parent_cristal'])
          parent = className.constantize.find_by(id_cristal: fields['id_parent_cristal'])
          if !already_updated_parent && !parent
            add_to_log_file("Updating parent structure #{fields['id_parent_cristal']}")
            parent_rex_entity = @client.search_structures({"fields.id_cristal": "|#{fields['id_parent_cristal']}"}).first
            add_to_log_file("Found parent rex")if parent_rex_entity
            parent = update_entity(parent_rex_entity, className, arbo_ids, true) if parent_rex_entity
          end
          structure.parent_id = parent && parent.id 
          add_to_log_file("Affected new parent #{structure.parent_id}") 
        end
      end
      
      add_to_log_file("Starting save structure: #{structure.id_cristal} changed : #{structure.changes}") 
      structure.save!
      add_to_log_file("End save structure: #{structure.id_cristal}") 

      if className == 'Hermes::Entity'
        if fields['itiephycrenational'] != structure.cre_national_id_cristal
          add_to_log_file("Start update cre nationnal with id #{fields['itiephycrenational']}")

          current_cre_national = Hermes::UserRight.find_by(type: 'entity', entity_id: structure.id, role: 'cre_national')
          add_to_log_file("Found existing cre national user right for this entity #{current_cre_national.id}") if current_cre_national

          if current_cre_national
            still_cre = false 
            
            if structure.cre_national_id_cristal
              contact_rex_json = @client.search_contacts({"fields.id_cristal": "|#{structure.cre_national_id_cristal}"}).first
              contact_rex_json.stringify_keys!
              still_cre = !!contact_rex_json && contact_rex_json['fields'].stringify_keys['id_etablissement_cristal'].include?(structure.id_cristal)
            end 
            
            if still_cre
              add_to_log_file("Downgraing old user right => cre local")
              current_cre_national.update(role: 'cre')
            else
              add_to_log_file("Removing old cre national")
              current_cre_national.destroy
            end 
          end  

          if fields['itiephycrenational'].blank?
            add_to_log_file("Clearing cre national fields")
            structure.update(
              cre_national_id_cristal: nil,
              cre_national_first_name: nil,
              cre_national_last_name: nil,
              cre_national_email: nil,
            )
          else 

            unless user_cre_national = User.find_by(id_cristal: fields['itiephycrenational'])
              add_to_log_file("User not found with id cristal")
              if user_cre_national = User.find_by(email: fields['ladrmailcrenational'], id_cristal: nil) 
                add_to_log_file("User found with email #{user_cre_national.id}")
                user_cre_national.update(id_cristal: fields['itiephycrenational'])
              else
                add_to_log_file("Creating new user")
                email = fields['ladrmailcrenational'].downcase
                user_cre_national = User.create!({
                  username: email,
                  first_name: fields['lprenomcrenational'],
                  last_name: fields['lnomcrenational'],
                  id_cristal: fields['itiephycrenational'],
                  email: email,
                  login: email,
                  auth_roles: ['cre'] ,
                  phone_number: "1234567890",
                  provider: HERMES_CONFIG['identity_provider_key'],
                  create_user_data: {
                    gender: 'M',
                    firstName: fields['lprenomcrenational'],
                    lastName: fields['lnomcrenational'],
                    email: email,
                    login: email,
                    mobileNumber: "1234567890",
                    roles: ['cre']
                  }
                }.merge(password: Devise.friendly_token[0,20]))
              end 
            end 

            if user_right_local = Hermes::UserRight.find_by(type: 'entity', entity_id: structure.id, user_id: user_cre_national.id)
              add_to_log_file("Removing old local user right #{user_right_local}")
              user_right_local.destroy
            end 

            add_to_log_file("Creating user right for new cre national")
            Hermes::UserRight.create!(
              type: 'entity', 
              entity_id: structure.id, 
              role: 'cre_national',
              right: 'administrator',
              user_id: user_cre_national.id,
              origin: 'rex',
            )

            add_to_log_file("Updating entity cre national fields")
            structure.update(
              cre_national_id_cristal: fields['itiephycrenational'],
              cre_national_first_name: fields['lprenomcrenational'],
              cre_national_last_name: fields['lnomcrenational'],
              cre_national_email: fields['ladrmailcrenational'],
            )
          end

          add_to_log_file("End update cre nationnal")
        end 
      end 

      structure
    end
    
    def update_contact(rex_entity)
      fields = rex_entity['fields']
      
      return if fields['email'].blank?
      add_to_log_file("updating #{fields['email']} #{fields}")
      
      if Rails.env.test? || @yopmail
        fields['email'] = fields['email'].split('@')[0] + '@yopmail.com'
      end
      
      contact = User.find_by(id_cristal: fields['id_cristal']) unless fields['id_cristal'].blank?
      contact = User.find_by(email: fields['email'].downcase) unless contact 
      
      if !contact
        add_to_log_file('Creating new user')
        contact = User.create(user_fields(fields).merge(password: Devise.friendly_token[0,20]))
      else
        add_to_log_file('Updating existing user')
        contact.skip_confirmation_notification!
        contact.update(user_fields(fields, contact.login, contact.auth_roles))
      end
      
      # Verifier qu'il n'y a pas un autre contact rex avec le même id_cristal car le rex peut avec 2 contact avec le même id_cristal 1 pour chaque role
      # dans ce cas on merge les entreprises et bailleur des 2 pour eviter de supprimer des habilitations
      unless Rails.env.test? 
        data = @client.search_contacts({'fields.id_cristal' => "|#{fields['id_cristal']}"})
        if data.length > 1
          [
            'id_etablissement_cristal',
            'id_bailleur_cristal',
          ].each do |rex_field_name|
            fields[rex_field_name] = data.reduce([]) do |array, b| 
              rex_id_cristal = b['fields'][rex_field_name]
              rex_id_cristal = [] if rex_id_cristal.blank?
              rex_id_cristal = [rex_id_cristal] if rex_id_cristal.is_a? String 
              array += rex_id_cristal
            end
          end 
        end 
      end

      update_user_right(fields: fields, type: 'entity', contact: contact)
      update_user_right(fields: fields, type: 'lessor', contact: contact)
    end

    private 

    def send_email(doc)

      support = User.find_by(:email=> "support@plugandwork.fr")
      current_time = DateTime.current.strftime('%Y-%m-%d-%H-%M-%S')

      error_length = @client.errors&.length

      doc_mail = Doc.create(
        responsible: support.id,
        type: 'email', 
        title: "Rapport Synchronisation REX #{HERMES_CONFIG['environment']} : #{current_time} #{error_length} erreurs", 
      )
      doc_mail.superfields['mail_to'] = (@email_mail_to).join(',')
      doc_mail.superfields['mail_from'] = support.email

      body = "Bonjour, <br/>"
      body += "Voici le lien vers le log de la syncro REX : <a style=\"font-weight: bold;color: #15c !important; \" href=\"https://#{APP_CONFIG["app_host"]}/documents/#{doc.id}\">#{doc.title}</a><br/>"
      body += "Nombre d'erreurs : #{error_length} <br/>"

      if error_length > 0
        body += "Les #{[100, error_length].min} premières erreurs : <br/>"
        @client.errors.take(100).each do |error|
          body += "- #{error}<br/>"
        end 
          
        if error_length > 100
          body += "Pour voir les erreurs suivantes, il faut aller sur l'admin PAW.<br/>"
        end 
      else
        body += "Aucune erreur.<br/>"
      end 

      doc_mail.body = body
      doc_mail.save

      # Envoi du mail
      Agent.regular_mail(doc_mail.id.to_s, 
        to: doc_mail.superfields['mail_to'], 
        from: doc_mail.superfields['mail_from'], 
        title: doc_mail.title, 
        content: doc_mail.body
      ).deliver_later
    end 

    def generate_structure_fields(fields , id)      
      {
        title: fields['app_comm'],
        id_rex: id,
        id_cristal: fields['id_cristal'],
        id_parent_cristal: fields['id_parent_cristal'],
        siret: fields['siret'].length == 14 ? fields['siret'] : nil,
      }
    end
    
    def update_user_right(options)
      type = options[:type]
      fields = options[:fields]
      contact = options[:contact]

      case type
      when  'entity'
        fieldname = 'id_etablissement_cristal'
        classname = Hermes::Entity
      when 'lessor'
        fieldname = 'id_bailleur_cristal'
        classname = Hermes::Lessor
      else
        raise 'update_user_right type not found'
      end

      return unless fields[fieldname]

      query = fields[fieldname].is_a?(String) ? fields[fieldname] : {'$in': fields[fieldname]}

      id_cristal_manager_group_ids = classname.where(id_cristal: query).pluck(:manager_group_id).map(&:to_s)
      user_right_group_ids = Hermes::UserRight.where(user_id: contact.id, type: type, right: 'administrator').pluck(:group_id)
      cre_national_group_ids = Hermes::UserRight.where(user_id: contact.id, type: type, right: 'administrator', role: 'cre_national').pluck(:group_id)

      added_ids = id_cristal_manager_group_ids - (user_right_group_ids - cre_national_group_ids )
      removed_ids = (user_right_group_ids + cre_national_group_ids ) - id_cristal_manager_group_ids

      added_user_rights = 0
      removed_user_rights = 0
      updated_user_rights = 0

      user_right_data = {
        user_id: contact.id, 
        type: type, 
        right: 'administrator', 
        origin: 'rex',
        role: fields['rolecrelocal'].blank? ? 'cre' : fields['rolecrelocal'].downcase
      }

      added_ids.each do |manager_group_id|
        next unless e = classname.find_by(manager_group_id: manager_group_id)
        if type == 'entity' && fields['rolecrelocal'].downcase == 'cre'
          e.set(updated_at: DateTime.current, cre_email: contact.email, cre_first_name: fields['prenom'], cre_last_name: fields['nom']) 
        end
        next if cre_national_group_ids.include?(manager_group_id)
        begin
          if ur = Hermes::UserRight.find_by(user_id: contact.id, type: type, "#{type}_id": e.id)
            ur.update(user_right_data.merge("#{type}_id": e.id))
            updated_user_rights += 1
          else 
            Hermes::UserRight.create(user_right_data.merge("#{type}_id": e.id))
            added_user_rights += 1
          end 
        rescue => exception
          puts "error: #{exception}"
        end
      end 

      removed_ids.each do |manager_group_id|
        next unless e = classname.find_by(manager_group_id: manager_group_id)
        if type == 'entity' && fields['rolecrelocal'].downcase == 'cre' && e.cre_email == contact.email
          e.set(updated_at: DateTime.current, cre_email: nil, cre_first_name: nil, cre_last_name: nil)
        end 
        next if cre_national_group_ids.include?(manager_group_id)
        if ur = Hermes::UserRight.find_by(user_right_data.merge("#{type}_id": e.id))
          ur.destroy
          removed_user_rights += 1
        end 
      end 

      add_to_log_file "Added #{added_user_rights}, updated #{updated_user_rights} and removed #{removed_user_rights} user_rights"
    end 

    def user_fields(fields, contact_login = nil, prev_auth_roles = [])
      mobileNumber = fields['tel_pro'].blank? ? '1234567890' : fields['tel_pro']
      role = fields['rolecrelocal'].blank? ? 'cre' : fields['rolecrelocal'].downcase
      auth_roles = ((prev_auth_roles  || []) + ['collaborateur offre logement'] + [role]).uniq
      login = contact_login || fields['email'].downcase
      {
        username: login,
        first_name: fields['prenom'],
        last_name: fields['nom'],
        id_cristal: fields['id_cristal'],
        email: fields['email'].downcase,
        login: login,
        auth_roles: auth_roles ,
        phone_number: mobileNumber,
        provider: HERMES_CONFIG['identity_provider_key'],
        create_user_data: {
          gender: 'M',
          firstName: fields['prenom'],
          lastName: fields['nom'],
          email: fields['email'].downcase,
          login: login,
          mobileNumber: mobileNumber,
          roles: auth_roles
        },
      }
    end
  end
end