Hermes::Engine.routes.draw do
  
  namespace :api do
    namespace :dmo do
      resources :housing_request_shares, only: :create do
        collection do
          get :find_by_housing_request_id
        end
      end
      resources :housing_request_shares, only: :show, controller: '/hermes/api/housing_request_shares' # no authenticated routes
      resources :applicants, only: [:index, :show, :update, :destroy]do
        member do
          get 'eligible_offers'
          post 'switch'

        end
        collection do
          get :find_by_nud
        end
      end

      resources :accountings, only: :index
      resources :entity_validations
      resources :entity_reservations
      resources :entities, only: [:index, :show,]  do
        member do
          put 'switch_manual_mode'
          get 'user_rights'
          get 'housing_requests_count'
          put 'update_annual_stock'
        end
        collection do
          get 'admin_search'
        end
      end
      resources :housing_request_entity_form_levels
      resources :housing_request_entity_forms do
        get :actives, on: :collection
        get :show_request, on: :member
      end
      resources :entity_codes, only: [:index, :create, :destroy] do
        post :check_entity_code_validity, on: :collection
      end
      resources :lessor_objective_histories
      resources :housing_applications do
      end
      resources :untensed_areas

      post 'housing_applications/update_status', to: "housing_applications#update_status"

      resources :pictures, only: [:create, :destroy]
      resources :housing_offers do
        collection do
          get 'with_validated_applications/:id', action: 'with_validated_applications_for'
          get 'with_validated_applications'
          get 'reference_exist'
          get 'with_recount_eligible_requests_nb'
          post 'search'
          post 'resend_flux5_all_missing_offers'
          post 'with_recount_eligible_requests_nb_rights_filtered'
          get 'simulate_eligible_requests_count'
        end
        member do
          get 'eligible_requests'
          get 'entity_statuses'
          get 'selected_applications'
          post 'selected_applications', to: 'housing_offers#post_selected_applications'
          get "all_applications",  to: 'housing_offers#all_applications'
          get 'history'
          post 'add_housing_channel'
          post 'remove_housing_channel'
          post 'resend_flux5'
          post 'eligible_requests_rights_filtered'
          post 'all_applications_rights_filtered'
          get 'is_eligible_request/:als_id', action: "is_eligible_request"
          post 'pictures', to: "pictures#add_picture_to_offer"
          delete 'pictures/:picture_id', to: "pictures#delete_picture_from_offer"
          post 'generate_exchange_token'
          get 'with_exchange_token'
        end
      end
      generate_notifications_routes # Helper to generate all notifications routes
      resources :eligible_count_offers, only: [:index, :show]
      
      resources :signup_user_notification_preferences, only: [:create] do
        collection do
          post 'create_multiple_preferences'
        end
      end
      get 'signup_user_notification_preferences/user_notifications_list', to: "signup_user_notification_preferences#user_notifications_list"
      resources :housing_channels
      resources :housing_channel_settings
      resources :housing_offer_expiration_settings
      resources :public_housing_offers
      resources :users, only: [:index, :show] do
        post :invalidate_token, on: :collection  
        post :switch, on: :member        
      end
      resources :update_subscriptions, only: [:index]
      
      delete 'user_rights/all_from_user', to: 'user_rights#delete_all_from_user'
      resources :user_rights, only: [:show, :index, :create, :destroy, :update] do 
        collection do 
          get 'is_administrator_for_lessor'
          get 'is_administrator_for_entity'
          delete 'bulk', to: 'user_rights#delete_bulk'
          post 'bulk', to: 'user_rights#post_bulk'
          post 'search'
          post 'all_users_with_rights'

        end 
      end 
      resources :operations
      resources :management_areas
      resources :lessor_contacts
      resources :nud_providers
      resources :housing_request_initializers, only: [:show, :index] 

      get 'rex/root_elements', to: 'rex#root_elements'
      get 'rex/:id/subsidiaries', to: 'rex#subsidiaries'
      get 'rex/:id', to: 'rex#show', as: :show


      resources :housing_requests do
        collection do
          get 'clp_details'
          get 'admin_search'
          get 'get_rfr_settings'
          post 'send_message'
          post 'send_flux_thirty'
          post 'search'
          patch 'entity_validation'
        end
        member do
          get 'entity_note'
          get 'internal_note'
          put 'update_internal_note'
          put 'update_entity_note'
          get 'housing_request_folder'
          get 'entity_requests'
          get 'waiting_points'
          get 'spent_points'
          post 'add_points_to_housing_request'
          post 'housing_request_folder/change_confirmed', to: "housing_requests#change_confirmed"
          post 'add_housing_channel'
          post 'validate_housing_channel'
          post 'remove_housing_channel'
          post 'update_from_sne'
          post 'validate_code'
          post 'notify'
          post 'send_flux_thirty_sync'
          get 'xml_sopra_errors'
          post 'force_update_renewal_values'
          put 'update_next_renewal_date_set_by_sne'
          get 'eligible_offers'
          get 'is_eligible_offer/:offer_id', action: "is_eligible_offer"
          post 'cancel_request'
        end
      end

 
      get 'required_documents/:id/historical_status', to: "required_documents#historical_status"
      get 'required_documents/:id', to: "required_documents#show"

      resources :lessors do
        collection do
          get 'admin_search'
        end
        member do
          get 'all_objectives'
          get 'active_objectives'
          put 'switch_manual_mode'
          get 'current_operation_names'
          put :add_user
          put :remove_user
          get :rdsl
        end
      end

      resources :lessor_objectives
      resources :nuds do
        collection do
          get 'pending_for_current_user'
          get 'sne_department_list'
          get 'imhoweb_department_list'
        end
        member do
          post 'send_pin_code_on_mail'
          post 'send_new_pin_code'
          post 'authentify_with_pin_code'
        end
      end
      resources :thresholds, only: %i[index show]
      resources :contact_groups
      resources :statuses, only: %i[index show] do
        get :applications_history, on: :collection
      end

      namespace :public_forms do
        get 'captcha'
        post 'confirm'
      end

      resources :reservation_holders, only: %i[index show]
      resources :user_filters, only: %i[index show create destroy]
      
      get 'doc', to: "documentation#show_doc"
      get 'changelog', to: "documentation#show_changelog"
      get 'old-changelog', to: "documentation#show_old_changelog"

    end # end of /api/dmo
  end # end of /api

  namespace :admin do # /admin
    resources :thresholds
    resources :rfr_settings
    match :rfr_settings, controller: 'rfr_settings', action: 'index' , via: %i[get put]

    resources :reservation_holders 
    resources :bordering_towns, only: %i[index] do 
      collection do
        get 'update_from_file'
        post 'update_from_file'
      end
    end 


    resources :nud_providers, only: %i[index new edit create update destroy]
    resources :housing_requests, only: %i[index create destroy update], path: :housing_requests # /admin/housing_requests
    resources :untensed_areas, only: %i[index create destroy update], path: :areas do # /admin/areas
      collection do
        get 'download_csv'
      end
    end
    resources :xml_errors, only: %i[index], path: :xml_errors do # /admin/areas
      member do
        post 'replay'
      end
    end
  end # end of /admin

  get '/hermes/gedfile/*path', to: "geds#file", :format => false
end
