# frozen_string_literal: true

module Hermes
  module BoClient
    class RequestOrientationTask < BaseTask
      
      def initialize(options = {})
        @xsd = "ComplementDemande.xsd"
        super(options)
      end
      
      endpoint {'/ws/rest/als/demande/v1/completer'}
      
      task_name 'application'
      type_flux 'BIEN'
      mapping 'Hermes::BoClient::Mappings::HousingRequestMap'
      
      params *%i[
      refInterneDemande
      refInterneOffre
      ]
      
      def add_if_exists(b, item_name, mapping_name = nil, is_code = false)
        mapping_name ||= item_name
        mapping_value = mapping.send(mapping_name)
        
        b.send(item_name, is_code ? {"code" => mapping_value} : mapping_value) unless mapping_value == nil
      
      end
      
      def to_xml
        namespaces = {
          'xmlns:ns2' => "http://v1.als.ikos.webservice.immobilier.sopra/demande/infosComplementaireDemande"
        }
        
        
        builder = Nokogiri::XML::Builder.new(encoding: 'UTF-8') do |b|
          b['ns2'].infosComplementaireDemande(namespaces) do
            b.numUnique mapping.numUnique do
              b.parent.namespace = nil
            end
            b.refInterneDemande mapping.refInterne do
              b.parent.namespace = nil
            end
            b.liste_orientation do
              b.parent.namespace = nil
              mapping.codes_orientation_intiale.each do |code|
                b.code_orientation code
              end
            end
            b.date_orientation mapping.date_orientation do
              b.parent.namespace = nil
            end
          end
        end
        
        builder.to_xml
      end
    end
  end
end
