require 'rails_helper'
require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')

RSpec.describe BatchTasks::NewPublishedOfferNotification, type: :service do

  subject { described_class.new }

  describe 'create_new_offers_notifications' do
    let!(:offer1) {create(:published_offer, :with_eligible_requests, requests_count: 3, date_publication_start: DateTime.now - 2.hours)}
    let!(:offer2) {create(:published_offer, :with_eligible_requests, requests_count: 2, date_publication_start: DateTime.now() - 2.days, external_ref: 'offer2')}

    it 'works' do 
      offer1.eligibles_requests.first.set(active_application_id: 'test')
      expect do
        subject.generate_and_enqueue_notification(
          date_start: DateTime.now - 1.days, 
          date_end: DateTime.now,
        )
      end.to change { enqueued_jobs.select{|job| job[:job] == SlowNotifierJob }.length }.by 4

      async_notif_job = execute_last_job(SlowNotifierJob)
      expect(async_notif_job[:queue]).to eq('slow_notifications')
      expect(async_notif_job[:args][0]["datas"]["offer_ids"].sort).to eq([offer1.id.to_s].sort)

      email_job = execute_last_job(ActionMailer::DeliveryJob) 
      expect(email_job[:queue]).to eq('slow_notifications')
    end 

    it 'with offer_filter' do
      offer2.update(date_publication_start: offer1.date_publication_start)

      expect do
        subject.generate_and_enqueue_notification(
          date_start: DateTime.now - 1.days, 
          date_end: DateTime.now,
          offer_filter: {external_ref: offer2.external_ref.to_s}
        )
      end.to change { enqueued_jobs.select{|job| job[:job] == SlowNotifierJob }.length }.by 5
      last_job = execute_last_job(SlowNotifierJob)

      expect(last_job[:args][0]["datas"]["offer_ids"]).to eq([offer2.id.to_s])
    end 
    
  end 
end